<?php
// $Id

/**
 * @file
 * Enables keeping an easily and regularly updated web page or a blog.
 */

/**
 * Implementation of hook_node_info().
 */
function everyblog_node_info() {
	return array(
		'everyblog' => array(
			'name' => t('Blog entry'),
			'module' => 'everyblog',
			'description' => t('A <em>blog entry</em> is a single post to an online journal, or <em>blog</em>, each user can have multiple blogs.'),
		)
	);
}

/**
 * Impelmentaion of hook_init()
 *
 * Patterned after Blog Theme's hook_init(), this provides blog theme support to EveryBlog.
 */
function everyblog_init() {
	global $custom_theme;
	
	if (module_exists('blog_theme')) {
		$view = strtolower(arg(0));
		switch ($view) {
			case 'blog':
			    if (arg(1) != '') {
					$blog_id = arg(1);
					// Load the blog id, since $blog_id can be an actual blog id or the blog name
					if ( !is_numeric($blog_id) ) {
						$blog_name = urldecode($blog_id);
						$blog_name = str_replace('_', ' ', $blog_name);
						$query = "SELECT * FROM {everyblogs} WHERE blog_name LIKE '$blog_name'";
					}
					else {
						$query = "SELECT * FROM {everyblogs} WHERE blog_id = $blog_id";
					}
					$blog = db_fetch_object(db_query($query));
					$uid = $blog->uid;
			    }
				break;
				
			case 'blogs':
				if (strtolower(arg(1)) == 'user') {
					$uid = arg(2);
				}
				break;
		}

		if (isset($uid)) {
			$themes = list_themes();
			$account = user_load(
				array((is_numeric($uid) ? 'uid' : 'name') => $uid, 
				'status' => 1));
			// has the user specified a specific theme?
			if ( !empty($account->blog_theme) ) {
				// is the theme specified actuall enabled?
				if ($themes[$account->blog_theme]->status) {
					$custom_theme =	$account->blog_theme;
				}
				else {
					variable_get('theme_default', 'garland');
				}
			}
		}
	}
}

/**
 * Implementation of hook_perm().
 */
function everyblog_perm() {
	return array(
		'create blog',
		'create own blog entries',
		'edit own blog entries',
		'delete own blog entries',
		'create any blog entry',
		'edit any blog entry',
		'delete any blog entry',
		);
}

/**
 * Implementation of hook_access().
 */
function everyblog_access($op, $node, $account) {
	global $user;

	switch ($op) {
		case 'create':
			// Anonymous users cannot post even if they have the permission. 
			return user_access('create own blog entries', $account) ||
				user_access('create any blog entry', $account) && 
				($account->uid ? TRUE : NULL);
		case 'update':
			return user_access('edit any blog entry', $account) || 
				(user_access('edit own blog entries', $account) && 
				($node->uid == $account->uid)) ? TRUE : NULL;
		case 'delete':
			return user_access('delete any blog entry', $account) || 
				(user_access('delete own blog entries', $account) && 
				($node->uid == $account->uid)) ? TRUE : NULL;
	}
}

/**
 * Implementation of hook_form_alter()
 */
function everyblog_form_alter(&$form, $form_state, $form_id=NULL) {
	if (strpos($form_id, 'everyblog') > -1) {
		$vid = variable_get('everyblog_taxonomy_category', -1);
		if ($vid > -1) {
			$blog_category = db_fetch_object(db_query("SELECT name FROM vocabulary WHERE vid=$vid"));

			if (isset($form['taxonomy'])) {
				// remove the select box that Drupal automattically shows for taxonomy selection  for the blog category as
				// we are controlling this seperately
				$vocabs = $form['taxonomy'];
				foreach ($vocabs as $index=>$vocab) {
					if (is_array($vocab) && array_key_exists('#title', $vocab) && $vocab['#title'] = $blog_category->name) {
						$vocab_index = $index;
					}
				}
				
				if (isset($vocab_index)) {
					unset($form['taxonomy'][$vocab_index]);
				}
			}
		}
	}
}

/**
 * Implementation of hook_help().
 */
/*
function blog_help($path, $arg) {
}
*/

/**
 * Implementation of hook_form().
 */
function everyblog_form(&$node, $form_values) {
	global $nid, $user;
	
	$form['form_id']['#value'] = 'everyblog_form';

	$blog_id=(isset($_GET['blog']) ? (int) $_GET['blog'] : $node->blog_id);

	//
	// We need some JavaScript to handle hiding and unhiding the textbox
	// where the user can enter the name for a new blog, also, we want 
	// hint text to be clickcleared and clickrecalled
	drupal_add_js("
		function handleBlogNameVisibility(e) {
			var namebox = document.getElementById('my-blog-name-wrapper');
			var categorybox = document.getElementById('my-blog-category-wrapper');
			if( e.selectedIndex == 1 ) {
				namebox.style.display = '';
				categorybox.style.display = '';
			}
			else {
				namebox.style.display = 'none';
				categorybox.style.display = 'none';			
			}
		}
		
		function clickclear(thisfield, defaulttext) {
			if (thisfield.value == defaulttext) {
				thisfield.value = '';
			}
		}

		function clickrecall(thisfield, defaulttext) {
			if (thisfield.value == '') {
				thisfield.value = defaulttext;
			}
		}
	",
		 "inline"
	);
	
	$iid = isset($_GET['iid']) ? (int)$_GET['iid'] : 0;
	$type = node_get_types('type', $node);

	$uid = $user->uid;

	// Can the user create entries to any blog?
	if (user_access('create any blog entry', $user)) {
		// Since the user can add to any blog we need to list ALL the blogs
		$query = "SELECT blog_id, blog_name FROM {everyblogs}";
	}
	else {
		// Get a list of blogs that the current user has created
		$query = "SELECT blog_id, blog_name FROM {everyblogs} WHERE uid = $uid";
	}

	// Run the query to get the proper list of blogs
	$result = db_query($query);
	$blogs = array(
		'0' => t('Select a blog'),
		'-1' => t('Create a new blog'),
		);
	while ($blog = db_fetch_object($result)) {
		$blogs[$blog->blog_id] = $blog->blog_name;
	}
        
	// Store the user id in the form for quick recall later
	$form['everyblog_uid'] = array(
	    '#type'=>'value',
		'#value' => $uid);

	// This select box allows the user to select which blog they are creating an
	// entry for, or to create a new blog
	$form['blog_id'] = array(
		'#type' => 'select',
		'#title' => t('Blog'),
		'#default_value' => $blog_id,
		'#weight' => -5,
		'#required' => TRUE,
		'#options' => $blogs,
		'#attributes' => array(
			'onchange' => "handleBlogNameVisibility(this)",
			),
		);

	if (user_access('create blog')) { 
		// If the user has choosen to create a new blog this text box, which is hidden
		// using the #attributes by setting the style.display to 'none', allows the user
		// to name their new blog
		$display = "display:none;";
		
		if ($form_values['blog_id']['#value'] == -1) {
		    $display == '';
		}
		$form['blog_name'] = array(
			 '#type' => 'textfield',
			 '#title' => t('Name'),
			 '#description' => t('The name for your new blog'),
			 '#prefix' => "<span id='my-blog-name-wrapper' style='$display'>",
			 '#suffix' => ' </span>',
			 '#weight' => -4,		 
			# '#validate' => array('everyblog_validate_blogname'=>array())
			);
		
		$vid = variable_get('everyblog_taxonomy_category', -1);
		if ($vid > -1) {
			$raw_categories = taxonomy_get_tree($vid);
			foreach($raw_categories as $category) {
				$categories[$category->tid] = $category->name;
			}
			$form['blog_category'] = array(
				 '#type' => 'select',
				 '#default_value' => 0,
				 '#options' => $categories,
				 '#title' => t('Category'),
				 '#description' => t('Select the category for your new blog. ') .
				 	(user_access('administer taxonomy') ? t('<i>Adding new categories can be done by adding terms to the correct vocabulary</i>') : ''),
				 '#prefix' => "<span id='my-blog-category-wrapper' style='display:none;'>",
				 '#suffix' => ' </span>',
				 '#weight' => -3, 
				);
		}
		else {
			$form['blog_category'] = array(
				 '#type'=> 'markup',
				 '#title' => t('Category'),
				 '#value' => t('You need to pick a vocubulary to use as the categories for the blogs, this can be done in settings page for EveryBlog'),
				 '#prefix' => "<span id='my-blog-category-wrapper' style='display:none;'>",
				 '#suffix' => ' </span>',
				 '#weight' => -3, 
			);
		}
	}
	
	if (empty($node->body)) {
		// If the user clicked a "blog it" link, we load the data from the
		// database and quote it in the blog.
		
		if ($nid && $blog = node_load($nid)) {
			$node->body = '<em>'. $blog->body .'</em> ['. l($blog->blog_name, "node/$nid") .']';
		}

		if ($iid && $item = db_fetch_object(db_query(
			'SELECT 
				i.*, f.title as ftitle, f.link as flink 
			 FROM 
			 	{aggregator_item} i, {aggregator_feed} f 
			 WHERE 
			 	i.iid = %d AND 
			 	i.fid = f.fid', $iid))) {
			$node->title = $item->title;
			// Note: $item->description has been validated on aggregation.
			$node->body = '<a href="'. check_url($item->link) .'">'. check_plain($item->title) .'</a> - <em>'. $item->description .'</em> [<a href="'. check_url($item->flink) .'">'. check_plain($item->ftitle) ."</a>]\n";
		}

	}

	// The title for the blog entry
	$form['title'] = array(
		'#type' => 'textfield', 
		'#title' => check_plain($type->title_label), 
		'#required' => TRUE, 
		'#default_value' => !empty($node->title) ? $node->title : NULL, 
		'#weight' => -1);
		
	_everyblog_get_body_field($form, $node);
	
	return $form;
}

function everyblog_validate_blogname($form) {
    drupal_set_message('Form validated');
    
    $blog_id = (int) db_fetch_object(db_query(
        "SELECT blog_id FROM {everyblogs} WHERE blog_name = '%s'",
        $form['blog_name']['#value']));
                
    # if we found a blog_id then it already exists, which is bad
    if ( $blog_id > 0 ) {
        form_error($form, t('A blog with that name has already been created.'));
    }    
}

/**
 * Implementatoin of hook_insert()
 */
function everyblog_insert($node) {
	// A blog_id of -1 means the user choose to create a new blog
	if ($node->blog_id == -1) {
		// insert a everyblog record into the table to keep track of the
		// new blog
		db_query("INSERT INTO {everyblogs} SET blog_name='%s', uid='%s', tid='%s'", 
			$node->blog_name, 
			$node->uid,
			$node->blog_category);
		$blog_id = db_fetch_object(db_query('SELECT max(blog_id) AS blog_id FROM {everyblogs}'))->blog_id;
	}
	else {
		$blog_id = $node->blog_id;
	}
	
	// associate the newely created node with the blog it belongs to
	db_query("INSERT INTO {everyblogs_to_nid} SET blog_id='%s', nid='%s'", 
		$blog_id, 
		$node->nid);
}

/**
 * Implementation of hook_update() 
 */
function everyblog_update($node) {
	// A blog_id of -1 means the user choose to create a new blog
        if ($node->blog_id == -1) {
		// insert a everyblog record into the table to keep track of the
		// new blog
		db_query("INSERT INTO {everyblogs} SET blog_name='%s', uid='%s', tid='%s'",
			$node->blog_name,
			$node->uid,
		$node->blog_category);
		$blog_id = db_last_insert_id('everyblogs', 'blog_id');
	}
	else {
		$blog_id = $node->blog_id;
	}

	// update the blog matching
	db_query("UPDATE {everyblogs_to_nid} SET blog_id='%s' WHERE nid='%s'",
		$blog_id,
		$node->nid);
}

/**
 * Implementation of hook_view().
 */
function everyblog_view($node, $teaser = FALSE, $page = FALSE) {
	if ($page) {
		// Breadcrumb navigation
		drupal_set_breadcrumb(array(
			l(t('Home'), NULL), 
			l(t('Blogs'), 'blogs'), 
			l($node->blog_name, 'blog/'. $node->blog_id)
			));
	}
	return node_prepare($node, $teaser);
}

/**
 * Implementation of hook_load().
 */
function everyblog_load($node) {
	$additions = db_fetch_object(db_query('SELECT b.blog_id, b.blog_name FROM {everyblogs} AS b, {everyblogs_to_nid} AS b_to_n WHERE b_to_n.nid = %s AND b_to_n.blog_id = b.blog_id', $node->nid));
	
	return $additions;
}

/**
 * Implementation of hook_link().
 */
function everyblog_link($type, $node = NULL, $teaser = FALSE) {
	$links = array();

	if ($type == 'node' && $node->type == 'blog') {
		if (arg(0) != 'blog' || arg(1) != $node->uid) {
			$links['blog_usernames_blog'] = array(
				'title' => t("@username's blog", array('@username' => $node->name)),
				'href' => "blog/$node->uid",
				'attributes' => array('title' => t("Read @username's latest blog entries.", array('@username' => $node->name)))
			);
		}
	}

	return $links;
}

/**
 * Access callback for user blog pages.
 */
function everyblog_page_user_access($account) {
	// The visitor must be able to access the site's content.
	// For a blog to 'exist' the user must either be able to
	// create new blog entries, or it must have existing posts.
	return user_access('access content');
	//return $account->uid && user_access('access content') && (user_access('create blog entries', $account) || _everyblog_post_exists($account));
}

/**
 * Helper function to determine if a user has blog posts already.
 */
function _everyblog_post_exists($account) {
	return (bool)db_result(db_query_range(db_rewrite_sql("SELECT 1 FROM {node} n WHERE n.type = 'blog' AND n.uid = %d AND n.status = 1"), $account->uid, 0, 1));
}

/**
 * Implementation of hook_block().
 *
 * Displays the most recent 10 blog titles.
 */
function everyblog_block($op = 'list', $delta = 0) {
	global $user;
	if ($op == 'list') {
		$block[0]['info'] = t('Recent blog posts');
		return $block;
	}
	else if ($op == 'view') {
		if (user_access('access content')) {
			$result = db_query_range(db_rewrite_sql("SELECT n.nid, n.title, n.created FROM {node} n WHERE n.type = 'everyblog' AND n.status = 1 ORDER BY n.created DESC"), 0, 10);
			if ($node_title_list = node_title_list($result)) {
				$block['content'] = $node_title_list;
				$block['content'] .= theme('more_link', url('blog'), t('Read the latest blog entries.'));
				$block['subject'] = t('Recent blog posts');
				return $block;
			}
		}
	}
}

function everyblog_admin() {
	$vocabs = taxonomy_get_vocabularies();

	$options[-1] = '-None-';
	foreach($vocabs as $vocab) {
		$options[$vocab->vid] = $vocab->name;
	}
	
	$form['everyblog_taxonomy_category'] = array(
		'#type' => 'select',
		'#title' => t('Blog Taxonomy Catogory'),
		'#description' => t('The vocabulary to use for categorizing blogs.'),
		'#default_value' => variable_get('everyblog_taxonomy_category', 'Blog Catagories'),
		'#options' => $options
	);
	
	$form['everyblog_show_datetime'] = array(
		'#type' => 'checkbox',
		'#title' => t('Show date/time'),
		'#description' => t('Show the date and time beside the titles when listing blogs.'),
		'#default_value' => variable_get('everyblog_show_datetime', FALSE),
	);
	
	return system_settings_form($form);
}

function everyblog_migrate($form_state) {
	if (is_array($form_state) && (array_key_exists('post', $form_state) && !array_key_exists('op', $form_state['post']))){
		$form['t1'] = array(
			'#value' => t('This will migrate existing blogs from Durpal\'s core Blog module to EveryBlog.<br>'),
			);

		// group users together
		$form['users'] = array(
			'#type' => 'fieldset',
			'#title' => t('User\'s Blogs'),
			'#description' => t('Select the users whose blogs you would like to migrate'),
			);
			
		// get a list of users
		$result = db_query("SELECT n.uid, u.name FROM {node} AS n, {users} AS u WHERE n.type = 'blog' AND u.uid = n.uid GROUP BY uid ORDER BY u.name ");
		$i = 0;
		while ($user = db_fetch_object($result)) {			
			$form['users']["u$i"] = array(
				'#type' => 'checkbox',
				'#title' => $user->name,
				'#default_value' => 1,
				'#return_value' => $user->uid . ':' . $user->name,
				);
			$i += 1;
		}
		$form['users']['numusers'] = array(
			'#type' => 'hidden',
			'#value' => $i,
			);
		
		$form['blog_name'] = array(
			'#type' => 'textfield',
			'#title' => t('Blog names'),
			'#value' => t('%u\'s blog'),
			'#description' => t('The name to give the blogs when they are migrated. \'%u\' is replaced with the user\'s name'),
			);
		
		$form['submit'] = array(
			'#type'=>'submit',
			'#value'=>'Migrate',
			);
	}
	else {
		$blog_name_template = $form_state['post']['blog_name'];
		if ($blog_name_template == '') {
			drupal_set_message('Problem migrating blogs, the blog name template was blank. Please try again.', 'error');
			return array();
		}
	
		$form['t1'] = array(
			'#value' => t('Existing blogs from Durpal\'s core Blog module have been migrated to EveryBlog.<br>'),
			);

		// group users together
		$form['users'] = array(
			'#type' => 'fieldset',
			'#title' => t('Migrated Blogs'),
			);
			
		// get a list of users
		$numusers = $form_state['post']['numusers'];
		for ($i =0; $i < $numusers; $i++) {
			// make sure the user was selected
			if (!array_key_exists("u$i", $form_state['post'])) continue;
			
			// split up the returned value into the user id and user name
			list($uid, $name) = split(':', $form_state['post']["u$i"]);
			
			// turn the blog name template into the actual blog name
			$blog_name = str_replace('%u', $name, $blog_name_template);
			
			// create the new blog
			db_query("INSERT INTO {everyblogs} SET blog_name='%s', uid=%d, tid=0", $blog_name, $uid);
			$blog_id = db_last_insert_id('everyblogs', 'blog_id');
			
			// associate the nodes with the new blog
			db_query("INSERT INTO {everyblogs_to_nid} (blog_id, nid) SELECT %d AS blog_id, n.nid FROM {node} AS n WHERE n.type = 'blog' AND n.uid = %d", $blog_id, $uid);
			
			// update the node type
			db_query("UPDATE {node} SET type = 'everyblog' WHERE type='blog' AND uid=%d", $uid);
			
			$blogs[] = t('@name to @blogname (@numentries entries)', array('@name'=>$name, '@blogname'=>$blog_name, '@numentries'=> db_affected_rows()));
		}
		$form['users']['list'] = array('#value'=>theme_item_list($blogs));
	}
	return $form;
}


